<?php

namespace Drupal\codit_menu_tools;

use Drupal\Core\Menu\MenuTreeParameters;

/**
 * Base class for Menu manipulation.
 */
abstract class MenuManipulatorBase implements MenuManipulatorBaseInterface {

  /**
   * The machine name of the menu.
   *
   * @var string
   */
  protected $menuName;

  /**
   * The menu storage service.
   *
   * @var \Drupal\Core\Entity\ContentEntityStorageInterface
   */
  protected $menuStorage;

  /**
   * The menu storage service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeElement[]
   */
  protected $menuTree;

  /**
   * Constructor for MenuManipulatorBase.
   *
   * @param string $menu_name
   *   The name of the menu to load.
   */
  public function __construct(string $menu_name) {
    $this->getMenuTree($menu_name);
    // Can't use it with Dependency Injection because runs from script or
    // hook_update_N where DI is not available.
    $this->menuStorage = \Drupal::entityTypeManager()->getStorage('menu_link_content');
  }

  /**
   * {@inheritdoc}
   */
  public function getMenuTree(string $menu_name = '') {
    if (empty($menu_name) && empty($this->menuTree)) {
      throw new \Exception('MenuManipulatorBase::getMenuTree must have a menu name to load.');
    }
    if (empty($this->menuTree)) {
      // Try to load the menu since it has not been loaded previously.
      $this->menuName = $menu_name;
      $tree = \Drupal::menuTree()->load($menu_name, new MenuTreeParameters());
      if (empty($tree)) {
        throw new \Exception("The Menu manipulator found no menu by the name '$menu_name'.");
      }
      $this->menuTree = $tree;
    }

    return $this->menuTree;
  }

  /**
   * Validate that the title and destination are provided.
   *
   * @param string $item_title
   *   The title of the intended menu item.
   * @param string $item_destination
   *   The destination of the intended menu item.
   *
   * @throws \Exception
   *   If any of the required menu details are not provided.
   */
  protected function preCheckRequiredItemValues(string $item_title, string $item_destination) : void {
    // Should throw exception if any of these values are empty.
    if (empty($item_title) && empty($item_destination)) {
      throw new \Exception("In order for MenuManipulatorBase to add a menu item, it must have a title and destination.");
    }
    elseif (empty($item_title)) {
      throw new \Exception("In order for MenuManipulatorBase to add a menu item, it must have a title.");
    }
    elseif (empty($item_destination)) {
      throw new \Exception("In order for MenuManipulatorBase to add a menu item, it must have a destination.");
    }
  }

  /**
   * {@inheritdoc}
   */
  public function findMenuItemDetailsByName(string $menu_item_title = '', $parent_id = ''): array {
    $item_details = [];
    if (!empty($menu_item_title)) {
      $query = $this->menuStorage->getQuery()
        ->condition('menu_name', $this->menuName)
        ->condition('title', $menu_item_title, '=');
      if (!empty($parent_id)) {
        // There is a parent id, use it for more certainty.
        $query->condition('parent', $parent_id, '=');
      }
      // We are just looking for something, no need for access check.
      $query->accessCheck(FALSE);
      $mids = $query->execute();

      $mids = array_values($mids);
      // Grab the last instance in case multiple exist.
      // This is a little risky, but it is an edge case.
      $mid = end($mids);
      /** @var Drupal\menu_item_extras\Entity\MenuItemExtrasMenuLinkContent $menuItem */
      if ($menuItem = $this->menuStorage->load($mid)) {
        $item_details['mid'] = $mid;
        $item_details['id'] = "menu_link_content:{$menuItem->uuid->value}";
        // Send back a 0 for weight as a neutral failsafe if none is recorded.
        $item_details['weight'] = (empty($menuItem->getWeight())) ? 0 : $menuItem->getWeight();
        // Use the sybling parent id rather than what was passed in.
        // What was passed in was just to make the search more precise.
        $item_details['parent_id'] = $menuItem->getParentId();
      }
    }

    return $item_details;
  }

}
